package calculatecv;

import com.csvreader.CsvReader;
import java.io.File;
import java.util.ArrayList;

/**
 * A CSVFile is a File but specifically of type CSV.
 * Provides functions that are specific to CSV file structure. For instance.
 * extracting a specific column from a file is different depending on whether
 * the file is CSV or XLS or XLSX.
 * 
 * @author Robert Eisinger
 */
public class CSVFile extends File {

    private CsvReader reader;
    private boolean hasHeaders;
    private String[] headers;
    private ArrayList<String> exp;
    private ArrayList<String> obs;

    public CSVFile(String pathname) {
        super(pathname);
        try {
            reader = new CsvReader(pathname);
            reader.readHeaders();
            headers = reader.getHeaders();
        } catch (Exception ex) {
            ex.printStackTrace();
        }

    }

    public void setHasHeaders(boolean b) {
        hasHeaders = b;
        if (hasHeaders) {
            getHeaders();
        }
    }

    public String[] getHeaders() {
        return headers;
    }

    public int numberOfColumns() {
        return headers.length;

    }

    public ArrayList<String> getColumn(int i) {
        try {
            ArrayList<String> output = new ArrayList<String>();

            reader = new CsvReader(getAbsolutePath());
            if (Config.input.hasHeaders()) {
                reader.readHeaders();
            }

            while (reader.readRecord()) {
                output.add(reader.getValues()[i]);
            }

            return output;

        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }

    public int posOfHeader(String h) {
        for (int i = 0; i < headers.length; i++) {
            if (headers[i].equals(h)) {
                return i;
            }
        }
        return -1;
    }

    public ArrayList<String> getColumnByHeader(String header) {
        return getColumn(posOfHeader(header));
    }

    public ArrayList<String> getColumnByColumnAlphabet(String alpha) {
        return getColumn(posOfColumnInColumnAlphabet(alpha));
    }

    public int posOfColumnInColumnAlphabet(String alpha) {
        String alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

        ArrayList<String> all = new ArrayList<String>();
        for (int i = 0; i < alphabet.length(); i++) {
            all.add(alphabet.charAt(i) + "");
        }

        for (int i = 0; i < alphabet.length(); i++) {
            for (int j = 0; j < alphabet.length(); j++) {
                all.add(alphabet.charAt(i) + "" + alphabet.charAt(j) + "");
            }
        }

        return all.indexOf(alpha);
    }

    public String validateExpectedObserved() {

        if (Config.input.hasHeaders()) {
            exp = getColumnByHeader(Input.expectedColumn);
            obs = getColumnByHeader(Input.observedColumn);
        } else {
            exp = getColumnByColumnAlphabet(Input.expectedColumn);
            obs = getColumnByColumnAlphabet(Input.observedColumn);
        }

        return Input.validateExpectedObserved(exp, obs);
    }

    public ArrayList<String> getExp() {
        return exp;
    }

    public ArrayList<String> getObs() {
        return obs;
    }

}
