package calculatecv;

import R.R;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import javax.swing.JOptionPane;

/**
 * The GoThread initiates all the R code and calls the appropriate Java methods
 * that interface with the R code, depending on what model options the user
 * selected.
 * 
 * @author Robert Eisinger
 */
public class GoThread implements Runnable {

    //Force a pause in between the initial few steps of the GoThread
    private static final int PAUSE = 0;
    
    //The error that resulted, if any
    public static String error;
    
    //Keep running while this value is true
    public static boolean run = true;
    
    /**
     * End running, usually if the user hit "Close"
     */
    public void end() {
        run = false;
    }

    @Override
    public void run() {
        System.out.println("run");
        boolean ready = runAll();
        if (!ready) {
            Go.loadingWindow.error(error);
        } else {
            Go.loadingWindow.success();
        }
    }

    /**
     * Does all the running of all the R, depending on what the user has 
     * selected in the GUI for model options.
     * 
     * @return true if running was successful
     */
    public static boolean runAll() {
        GoLoadingWindow loadingWindow = Go.loadingWindow;

        /* INITIATE */
        loadingWindow.addTextToLog("Initiating...");
        sleep(PAUSE);
        loadingWindow.addTextToLog("DONE");
        
        if(!run) {
            return true;
        }

        /* CHECK DEPENDENCIES */
        loadingWindow.addTextToLog("Checking dependencies (from Step 1)...", true);
        sleep(PAUSE);
        Config.step1Panel.checkDependencies();
        if (!Step1Panel.dependenciesSatisfied) {
            loadingWindow.addTextToLog("FAILED\nThe dependencies have not been fulfilled. Please return to Step 1.", LogEventType.FAILURE);
            return false;
        }
        loadingWindow.addTextToLog("SUCCESS", LogEventType.SUCCESS);

        if(!run) {
            return true;
        }

        /* CHECK DATA VALIDATION */
        loadingWindow.addTextToLog("Validating input data (from Step 2)...", true);
        sleep(PAUSE);
        Input input = Config.input;
        if (input == null) {
            loadingWindow.addTextToLog("FAILED\nYou haven't uploaded data. Please return to Step 2.", LogEventType.FAILURE);
            return false;
        }
        
        if(!run) {
            return true;
        }
        
        if (input.uploadFailed()) {
            loadingWindow.addTextToLog("FAILED\nYour data upload was unsuccessful. Please return to Step 2.", LogEventType.FAILURE);
            return false;
        }
        
        if(!run) {
            return true;
        }
        
        if (input.validationFailed()) {
            loadingWindow.addTextToLog("FAILED\nData validation failed. Please return to Step 2.\n" + input.getValidationMessage(), LogEventType.FAILURE);
            return false;
        }
        loadingWindow.addTextToLog("SUCCESS", LogEventType.SUCCESS);
        
        if(!run) {
            return true;
        }

        /* CHECK MODEL OPTIONS */
        loadingWindow.addTextToLog("Validating model options (from Step 3)...", true);
        sleep(PAUSE);

        //check that at least one model is selected
        Step3Panel panel = Config.step3Panel;
        if (!panel.powerIsSelected() && !panel.linearIsSelected() && !panel.logIsSelected()) {
            loadingWindow.addTextToLog("FAILED\nYou must select at least one of Power, Linear, or Log. Please return to Step 3.", LogEventType.FAILURE);
            return false;
        }
        
        if(!run) {
            return true;
        }
        
        //check that if outlier is selected, the box has a valid number
        if (panel.outlierIsSelected() && !panel.outlierTextFieldValidationl()) {
            loadingWindow.addTextToLog("FAILED\nOutlier bounds must be between 0.0 and 99.0. Please return to Step 3.", LogEventType.FAILURE);
            return false;
        }
        
        if(!run) {
            return true;
        }
        
        //check that if discontinuous is selected, the box has a valid number
        if (panel.discontinuousIsSelected() && !panel.discontinuousTextFieldValidation()) {
            loadingWindow.addTextToLog("FAILED\nThe point of discontinuity must be between 0.0 and 99.0. Please return to Step 3.", LogEventType.FAILURE);
            return false;
        }
        
        if(!run) {
            return true;
        }
        
        //check that if alpha, beta, or sigma is selected, the boxes have valid numbers
        if ((panel.alphaIsSelected() && !panel.alphaTextFieldValidation())
                || (panel.betaIsSelected() && !panel.betaTextFieldValidation())
                || (panel.sigmaIsSelected() && !panel.sigmaTextFieldValidation())) {
            loadingWindow.addTextToLog("FAILED\nWhen significance testing against prespecfied parameters, values for \u03B1, \u03B2, and \u03C3 must be between 0.0 and 99.0. Please return to Step 3.", LogEventType.FAILURE);
            return false;
        }
        loadingWindow.addTextToLog("SUCCESS", LogEventType.SUCCESS);

        if(!run) {
            return true;
        }
        
        /* CHECK OUTPUT OPTIONS */
        loadingWindow.addTextToLog("Validating output options (from Step 4)...", true);
        sleep(PAUSE);
        if (!Config.step4Panel.isOutputDirectoryADirectory()) {
            loadingWindow.addTextToLog("FAILED\nThe current output directory is not an existing directory. Please return to Step 4.", LogEventType.FAILURE);
            return false;
        }
        
        if(!run) {
            return true;
        }
        
        if (!Config.step4Panel.outputIsXLSX()) {
            loadingWindow.addTextToLog("FAILED\nThe output file must be xlsx and less than 60 characters long. Please return to Step 4.", LogEventType.FAILURE);
            return false;
        }

        if(!run) {
            return true;
        }
        
        //get ready to run
        Config.input.initExpectedObserved();
        R.init();

        /* GET MODEL PARAMETERS FOR EACH MODEL SELECTED */
        if (panel.powerIsSelected()) {
            loadingWindow.addTextToLog("\n\nEstimating parameters via power model...", true);
            boolean power = powerFit();
        }
        
        if(!run) {
            return true;
        }

        if (panel.linearIsSelected()) {
            loadingWindow.addTextToLog("\n\nEstimating parameters via linear model...", true);
            boolean linear = linearFit();
        }
        
        if(!run) {
            return true;
        }

        if (panel.logIsSelected()) {
            loadingWindow.addTextToLog("\n\nEstimating parameters via log model...", true);
            boolean log = logFit();
        }
        
        if(!run) {
            return true;
        }

        if (panel.countingIsSelected()) {
            loadingWindow.addTextToLog("\n\nEstimating parameters via counting model...", true);
            boolean counting = countingFit();
        }
        
        if(!run) {
            return true;
        }
        
        if (panel.discontinuousIsSelected() && panel.discontinuousTextFieldValidation()) {
            loadingWindow.addTextToLog("\n\nEstimating parameters via discontinuous model...", true);
            boolean dis = discontinuousFit();
        }
        
        if(!run) {
            return true;
        }

        /* CREATE PLOTS FOR EACH SUBJECT */
        if(Config.step4Panel.savePlotImagesIsSelected()) {
            loadingWindow.addTextToLog("\n\nGenerating plots...", true);
            R.generatePlots();
        }
        
        
        /* GET AIC COMPARISONS BETWEEN MODELS */
        if (panel.numModelsSelected() >= 2) {
            loadingWindow.addTextToLog("\n\nFinding AIC values...", true);
            boolean aic = AIC(panel.powerIsSelected(), panel.linearIsSelected(), panel.logIsSelected(), panel.discontinuousIsSelected(), panel.countingIsSelected());
        }
        
        if(!run) {
            return true;
        }

        if (panel.shouldTestAgainstPrespecifiedLikelihood()) {
            loadingWindow.addTextToLog("\n\nTesting for significance against prespecified parameters using a Likelihood Ratio Test...", true);
            boolean sig = likelihoodRatioTest(panel.alphaIsSelected(), panel.betaIsSelected(), panel.sigmaIsSelected());
        }
        
        if(!run) {
            return true;
        }
        
        if(panel.shouldTestAgainstPrespecifiedWald()) {
            loadingWindow.addTextToLog("\n\nTesting for significance against prespecified parameters using the Wald Test...", true);      
            boolean sig = waldTest();
        }
        
        if(!run) {
            return true;
        }
        
        /* BAYESIAN */
        BaysPanel bays = Config.baysPanel;
        System.out.println("BAYYYYYYYS: " + bays);
        if(bays!=null && bays.fitBayesianSelected()) {
            loadingWindow.addTextToLog(("\n\nRunning Bayesian fits."));
            
            if(panel.powerIsSelected()) {
                loadingWindow.addTextToLog("\n\nUsing Bayesian to estimate parameters via power model...",true);
                boolean powerBays = bayesPower();
            }
            
            if(panel.linearIsSelected()) {
                loadingWindow.addTextToLog("\n\nUsing Bayesian to estimate parameters via linear model...",true);
                boolean linearBayes = bayesLinear();
            }
            
            if(panel.logIsSelected()) {
                loadingWindow.addTextToLog("\n\nUsing Bayesian to estimate parameters via log model...",true);
                boolean logBayes = bayesLog();
            }
        }
        
        if(!run)
            return true;
        
        /* GET DIC COMPARISONS BETWEEN MODELS */
        if (panel.numModelsSelected() >= 2) {
            loadingWindow.addTextToLog("\n\nFinding DIC values...", true);
            boolean aic = DIC(panel.powerIsSelected(), panel.linearIsSelected(), panel.logIsSelected(), panel.discontinuousIsSelected(), panel.countingIsSelected());
        }

        Output.save();
        
        if(panel.numModelsSelected() > 1) {
            JOptionPane.showMessageDialog(loadingWindow, "Done! Note that each specified model will be in a seperate Excel sheet.");
        }
        
        
        

        return true;
    }
    
    /**
     * Runs a Wald Test for the data and options the user has selected
     * @return false if an error occurred, true otherwise
     */
    public static boolean waldTest() {
        LinkedHashMap<String, LinkedHashMap<String, ArrayList<String>>> out = R.waldTest();
        if (out == null) {
            return false;
        } else if (!out.isEmpty()) {
            Output.addToOutput(out);
        }

        return true;
    }

    /**
     * Runs a likelihood ratio test against prespecified values of one ore more
     * of alpha, beta, and sigma
     * @param alpha whether or not to test against alpha
     * @param beta whether or not to test against beta
     * @param sigma whether or not to test against sigma
     * @return false if an error occurred, true otherwise
     */
    public static boolean likelihoodRatioTest(boolean alpha, boolean beta, boolean sigma) {
        LinkedHashMap<String, LinkedHashMap<String, ArrayList<String>>> out = R.likelihoodRatioTest(alpha, beta, sigma);
        if (out == null) {
            return false;
        } else if (!out.isEmpty()) {
            Output.addToOutput(out);
        }

        return true;
    }

    /**
     * Finds the AIC differences by finding AIC values for each model that the 
     * user has selected
     * 
     * @param power whether or not the user has selected power
     * @param linear whether or not the use has selected linear
     * @param log whether or not the user has selected log
     * @param discontinuous whether or not the use has selected discontinuous
     * @param counting whether or not the user has selected counting
     * @return false if an error occurs, true otherwise
     */
    public static boolean AIC(boolean power, boolean linear, boolean log, boolean discontinuous, boolean counting) {
        LinkedHashMap<String, LinkedHashMap<String, ArrayList<String>>> out = R.AIC(power, linear, log, discontinuous, counting);
        if (out == null) {
            return false;
        } else if (!out.isEmpty()) {
            Output.addToOutput(out);
        }

        return true;
    }
    
    public static boolean DIC(boolean power, boolean linear, boolean log, boolean discontinuous, boolean counting) {
        LinkedHashMap<String, LinkedHashMap<String, ArrayList<String>>> out = R.DIC(power, linear, log, discontinuous, counting);
        if (out == null) {
            return false;
        } else if (!out.isEmpty()) {
            Output.addToOutput(out);
        }

        return true;
    }
    
    /**
     * Run a discontinuous fit
     * @return false if an error occurred, true otherwise
     */
    public static boolean discontinuousFit() {
        LinkedHashMap<String, ArrayList<String>> out = R.runDiscontinuousModel(Config.step3Panel.getDiscontinuousTextFieldValue());
        if (out == null) {
            return false;
        } else {
            Output.addToOutput("discontinuous", out);
        }

        return true;        
    }

    /**
     * Run a counting fit
     * @return false if an error occurred, true otherwise
     */
    public static boolean countingFit() {
        LinkedHashMap<String, ArrayList<String>> out = R.countingFit();
        if (out == null) {
            return false;
        } else {
            Output.addToOutput("counting", out);
        }

        return true;
    }

    /**
     * Run a power fit
     * @return false if an error occurred, true otherwise
     */
    public static boolean powerFit() {
        LinkedHashMap<String, ArrayList<String>> out = R.powerFit();
        if (out == null) {
            return false;
        } else {
            Output.addToOutput("power", out);
        }

        return true;
    }
    
    public static boolean bayesPower() {
        LinkedHashMap<String, ArrayList<String>> out = R.bayesPowerFit();
        if (out == null) {
            return false;
        } else {
            Output.addToOutput("power.bayes", out);
        }

        return true;
    }
    
    public static boolean bayesLinear() {
        LinkedHashMap<String, ArrayList<String>> out = R.bayesLinearFit();
        if (out == null) {
            return false;
        } else {
            Output.addToOutput("linear.bayes", out);
        }

        return true;
    }
    
    public static boolean bayesLog() {
        LinkedHashMap<String, ArrayList<String>> out = R.bayesLogFit();
        if (out == null) {
            return false;
        } else {
            Output.addToOutput("log.bayes", out);
        }

        return true;
    }

    /**
     * Run a linear fit
     * @return false if an error occurred, true otherwise
     */
    public static boolean linearFit() {
        LinkedHashMap<String, ArrayList<String>> out = R.linearFit();
        if (out == null) {
            return false;
        } else {
            Output.addToOutput("linear", out);
        }

        return true;
    }

    /**
     * Run a log fit
     * @return false if an error occurred, true otherwise
     */
    public static boolean logFit() {
        LinkedHashMap<String, ArrayList<String>> out = R.logFit();
        if (out == null) {
            return false;
        } else {
            Output.addToOutput("log", out);
        }

        return true;
    }
    
    public static boolean generatePlots() {
        R.generatePlots();
        return true;
    }

    /**
     * Pause the thread
     * @param milli how long to pause the read for, in milliseconds
     */
    public static void sleep(int milli) {
        try {
            Thread.sleep(milli);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
