package calculatecv;

/**
 * Retrieves info about the user's computer
 * 
 * @author Robert Eisinger
 */
public class Info {

    /**
     * Gets the operating system
     * @return the operating system
     */
    public static String getOS() {
        return System.getProperty("os.name");
    }

    /**
     * Determines if the computer is a Windows machine
     * @return whether or not the computer is windows
     */
    public static boolean isWindows() {
        return getOS().startsWith("Windows");
    }

    /**
     * Determines if the computer is a Mac
     * @return whether or not the computer is a Mac
     */
    public static boolean isMac() {
        return getOS().startsWith("Mac");
    }

    /**
     * Gets the user's location via user.home property
     * @return the user's home location (e.g., /Users/Robert/)
     */
    public static String getUserLocation() {
        String user = System.getProperty("user.home");
        String location = "";
        if (Info.isMac()) {
            location = user + "/Documents/";
        } else {
            location = user + "\\Documents\\";
            location = location.replaceAll("\\\\","\\\\\\\\");
        }
        return location;
    }
}
