package calculatecv;

import java.io.File;
import java.util.ArrayList;

/**
 * 
 * @author Robert Eisinger
 */
public class Input {

    //The location of the input file(s)
    private String location;
    private boolean isDirectory;
    private ArrayList<File> inputFiles;
    private boolean uploadFailed;
    private boolean validationFailed;
    private String validationMessage;
    private String type;
    private boolean hasHeaders;
    public static int worksheet = 0;
    public static int numWorksheets = -1;
    public static String expectedColumn;
    public static String observedColumn;
    public static ArrayList<ExpObs> expectedObserved;

    public Input(String absolutePath, boolean hasHeaders) {
        this.hasHeaders = hasHeaders;

        if (!new File(absolutePath).exists()) {
            uploadFailed = true;
            validationFailed = true;
            validationMessage = "The selected location does not exist on your computer.";
        } else {

            location = absolutePath;
            isDirectory = new File(absolutePath).isDirectory();

            if (isDirectory) {
                File in = new File(location);
                File[] files = in.listFiles();
                initFiles(files);
            } else {
                initFiles(new File[]{new File(absolutePath)});
            }
        }
    }
    
    public void initExpectedObserved() {
        expectedObserved = new ArrayList<ExpObs>();
        for(int i = 0; i < inputFiles.size(); i++) {
            File f = inputFiles.get(i);
            if(f instanceof CSVFile) {
                expectedObserved.add(new ExpObs(f.getName(),((CSVFile)f).getExp(),((CSVFile)f).getObs()));
            } else {
                expectedObserved.add(new ExpObs(f.getName(),((XLSFile)f).getExp(),((XLSFile)f).getObs()));
            }
        }
        
        //we don't need a global subject if there is only one subject
        if(expectedObserved.size() == 1)
            return;
        
        //make the global subject
        ArrayList<String> globalExp = new ArrayList<String>();
        ArrayList<String> globalObs = new ArrayList<String>();
        for(ExpObs eo : expectedObserved) {
            double[] exp = eo.getExp();
            double[] obs = eo.getObs();
            for(double d : exp) {
                globalExp.add(d+"");
            }
            for(double d : obs) {
                globalObs.add(d+"");
            }
        }
        
        //uncomment to include Group Data in all the output, including plots
        //expectedObserved.add(new ExpObs("GROUP DATA",globalExp,globalObs));
    }

    public String getType() {
        return type;
    }

    public String getValidationMessage() {
        return validationMessage;
    }

    public boolean validationFailed() {
        return validationFailed;
    }

    public boolean uploadFailed() {
        return uploadFailed;
    }

    public void setHasHeaders(boolean b) {
        hasHeaders = b;
    }

    public boolean hasHeaders() {
        return hasHeaders;
    }

    /**
     * Used if if the input is a directory
     */
    private void initFiles(File[] files) {

        boolean hasCSV = hasCSV(files);
        boolean hasXLS = hasXLS(files);

        if (!hasCSV && !hasXLS) {
            validationFailed = true;
            uploadFailed = true;
            validationMessage = "Your folder contains neither csv, xls, or xlsx files.";
            return;
        } else if (hasCSV && hasXLS) {
            validationFailed = true;
            uploadFailed = true;
            validationMessage = "Your folder contains both csv and xls/xlsx files. Please select a folder with only csv or only xls/xlsx file types. This problem may occur when a folder contains more than just data files.";
            return;
        } else if (hasCSV) {
            type = "csv";
            validationFailed = false;
            uploadFailed = false;
            inputFiles = new ArrayList<File>();
            for (int i = 0; i < files.length; i++) {
                if (extension(files[i]).equals("csv") && isValid(files[i])) {
                    inputFiles.add(new CSVFile(files[i].getAbsolutePath()));
                }
            }
            validationMessage = "Successfully uploaded " + inputFiles.size() + " csv file(s).";
        } else if (hasXLS) {
            type = "xls";
            validationFailed = false;
            uploadFailed = false;
            inputFiles = new ArrayList<File>();
            for (int i = 0; i < files.length; i++) {
                if ((extension(files[i]).equals("xls") || extension(files[i]).equals("xlsx")) && isValid(files[i])) {
                    inputFiles.add(new XLSFile(files[i].getAbsolutePath()));
                }
            }
            validationMessage = "Successfully uploaded " + inputFiles.size() + " xls/xlsx file(s).";
        }
        
        if(getHeaders().length == 0) {
            validationFailed = true;
            if(inputFiles.size() == 0) {
                validationMessage = validationMessage + "\n" + " --> The file you uploaded does not have any columns. ";
            } else {
                validationMessage = validationMessage + "\n" + " --> To use this application with a folder, you must use files with similar column headers. This problem often occurs when your a folder has more than just data files. Please select a folder with only the relevant data files.";
            }
            return;
        }


        if (expectedColumn == null) {
            expectedColumn = getHeaders()[0];
        }

        if (observedColumn == null) {
            observedColumn = getHeaders()[0];
        }

        if (expectedColumn.equals(observedColumn)) {
            validationFailed = true;
            validationMessage = validationMessage + "\n" + " --> 'Expected' and 'Observed' data columns are currently the same. Please select two different columns.";
            return;
        }

        //validate expected column and observed column
        for (int i = 0; i < inputFiles.size(); i++) {
            String out = null;
            if (type.equals("xls")) {
                XLSFile f = (XLSFile) inputFiles.get(i);
                out = f.validateExpectedObserved();
            } else {
                CSVFile f = (CSVFile) inputFiles.get(i);
                out = f.validateExpectedObserved();
            }

            if (out != null) {
                validationFailed = true;
                validationMessage = validationMessage + "\n" + " --> Validating " + inputFiles.get(i).getName() + ": " + out;
                return;
            }
        }

    }
    
    public boolean isValid(File f) {
        if(f.getName().substring(0,1).equals(".")) {
            return false;
        }
        
        return true;
    }

    public String[] getHeaders() {
        if (hasHeaders) {
            ArrayList<String> headers = new ArrayList<String>();

            File f = inputFiles.get(0);
            String[] singleHeaders;
            if (type.equals("csv")) {
                singleHeaders = ((CSVFile) f).getHeaders();
            } else {
                singleHeaders = ((XLSFile) f).getHeaders();
            }

            for (int p = 0; p < singleHeaders.length; p++) {
                String header = singleHeaders[p];
                
                boolean foundInAll = true;
                for (int i = 1; i < inputFiles.size(); i++) {
                    f = inputFiles.get(i);
                    String[] otherSingleHeaders;
                    if (type.equals("csv")) {
                        otherSingleHeaders = ((CSVFile) f).getHeaders();
                    } else {
                        otherSingleHeaders = ((XLSFile) f).getHeaders();
                    }

                    boolean found = false;
                    for (int j = 0; j < otherSingleHeaders.length; j++) {
                        if(header.equals(otherSingleHeaders[j])) {
                            found = true;
                            break;
                        }
                    }
                    
                    if(!found) {
                        foundInAll = false;
                        break;
                    }
                }
                
                if(foundInAll) {
                    headers.add(header);
                }
            }

            String[] out = new String[headers.size()];
            for (int i = 0; i < out.length; i++) {
                out[i] = headers.get(i);
            }

            return out;

        } else {
            //if it doens't have headers, just count the total number of columns
            int minNumCols = Integer.MAX_VALUE;
            for (int i = 0; i < inputFiles.size(); i++) {
                File f = inputFiles.get(i);
                int numCols = 0;
                if (type.equals("csv")) {
                    numCols = ((CSVFile) f).numberOfColumns();
                } else {
                    numCols = ((XLSFile) f).numberOfColumns();
                }

                if (numCols < minNumCols) {
                    minNumCols = numCols;
                }
            }

            return columnAlphabet(minNumCols);
        }
    }

    public String[] columnAlphabet(int n) {
        String[] out = new String[n];
        String alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

        ArrayList<String> all = new ArrayList<String>();
        for (int i = 0; i < alphabet.length() && i < n; i++) {
            all.add(alphabet.charAt(i) + "");
        }

        for (int i = 0; i < alphabet.length(); i++) {
            for (int j = 0; j < alphabet.length(); j++) {
                all.add(alphabet.charAt(i) + "" + alphabet.charAt(j) + "");
            }
        }

        for (int i = 0; i < n; i++) {
            out[i] = all.get(i);
        }

        return out;

    }

    private boolean hasCSV(File[] files) {
        for (int i = 0; i < files.length; i++) {
            if (extension(files[i]).equals("csv")) {
                return true;
            }
        }
        return false;
    }

    private boolean hasXLS(File[] files) {
        for (int i = 0; i < files.length; i++) {
            if (extension(files[i]).equals("xls") || extension(files[i]).equals("xlsx")) {
                return true;
            }
        }
        return false;
    }

    public String extension(File f) {
        String s = f.getAbsolutePath();
        int index = s.lastIndexOf(".");
        return s.substring(index + 1, s.length()).toLowerCase();
    }

    public String[] getWorksheets() {
        if (numWorksheets == -1) {
            XLSFile f = (XLSFile) this.inputFiles.get(0);
            numWorksheets = f.getNumWorkseheets();
        }

        String[] out = new String[numWorksheets];
        for (int i = 0; i < out.length; i++) {
            out[i] = (i + 1) + "";
        }

        return out;
    }

    public static String validateExpectedObserved(ArrayList<String> exp, ArrayList<String> obs) {
        System.out.println(exp);
        System.out.println(obs);
        if (exp == null || obs == null) {
            return "Unable to read data. This could happen because your data is corrupted.";
        } else if (exp.size() != obs.size()) {
            return "'Expected' and 'Observed' columns are not of the same length.";
        } else {
            boolean allValuesAreTheSame = true;
            for (int i = 0; i < exp.size(); i++) {
                String e = exp.get(i);
                String o = obs.get(i);
                try {
                    Double.parseDouble(e);
                } catch (Exception q) {
                    //return "'Expected' column contains an invalid entry. Only numbers are allowed. Invalid cell value: '" + e + "'";
                    exp.remove(i);
                    obs.remove(i);
                    continue;
                }

                try {
                    Double.parseDouble(o);
                } catch (Exception q) {
                    //return "'Observed' column contains an invalid entry. Only numbers are allowed. Invalid cell value: '" + o + "'";
                    exp.remove(i);
                    obs.remove(i);
                    continue;
                }

                if (Double.parseDouble(e) != Double.parseDouble(o)) {
                    allValuesAreTheSame = false;
                }
            }

            if (allValuesAreTheSame) {
                return "'Expected' and 'Observed' columns appear to have the same exact values. MLE will fail under these conditions.";
            }
        }

        return null;
    }

    public ArrayList<ExpObs> getExpectedObserved() {
        return expectedObserved;
    }
}
