/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package calculatecv;

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

/**
 *
 * @author Robert
 */
public class XLSFile extends File {

    private String type;
    private int worksheet;
    private String[] headers;
    private ArrayList<String> exp;
    private ArrayList<String> obs;

    public XLSFile(String absolutePath) {
        super(absolutePath);
        if (absolutePath.endsWith("xls")) {
            type = "xls";
        } else {
            type = "xlsx";
        }

        worksheet = Input.worksheet;

    }

    public String getAsString(Cell c) {
        int t = c.getCellType();
        if (t == Cell.CELL_TYPE_BOOLEAN) {
            return c.getBooleanCellValue() + "";
        } else if (t == Cell.CELL_TYPE_NUMERIC) {
            return c.getNumericCellValue() + "";
        } else if (t == Cell.CELL_TYPE_STRING) {
            return c.getStringCellValue() + "";
        } else {
            return "";
        }
    }

    public int numberOfColumns() {
        if (headers == null) {
            initHeaders();
        }

        return headers.length;
    }

    public void initHeaders() {
        Iterator<Row> rowIterator = null;
        if (type.equals("xls")) {
            try {
                FileInputStream file = new FileInputStream(getAbsolutePath());
                HSSFWorkbook workbook = new HSSFWorkbook(file);
                HSSFSheet sheet = workbook.getSheetAt(Input.worksheet);
                rowIterator = sheet.iterator();

            } catch (Exception e) {
                e.printStackTrace();
            }

        } else if (type.equals("xlsx")) {
            try {
                FileInputStream file = new FileInputStream(getAbsolutePath());
                XSSFWorkbook workbook = new XSSFWorkbook(file);
                XSSFSheet sheet = workbook.getSheetAt(Input.worksheet);
                rowIterator = sheet.iterator();
            } catch (Exception e) {
                System.err.println("Error in file: " + this.getName());
                e.printStackTrace();
            }
        }

        ArrayList<String> heads = new ArrayList<String>();
        Row row = rowIterator.next();
        Iterator<Cell> cellIterator = row.cellIterator();

        int count = 0;
        while (cellIterator.hasNext()) {
            Cell c = cellIterator.next();
            heads.add(getAsString(c));
            count++;
        }

        headers = new String[heads.size()];
        for (int i = 0; i < heads.size(); i++) {
            headers[i] = heads.get(i);
        }
    }

    public String[] getHeaders() {
        if (headers == null) {
            initHeaders();
        }
        return headers;
    }

    public int getNumWorkseheets() {
        if (type.equals("xls")) {
            try {
                FileInputStream file = new FileInputStream(getAbsolutePath());
                HSSFWorkbook workbook = new HSSFWorkbook(file);
                return workbook.getNumberOfSheets();
            } catch (Exception e) {
                e.printStackTrace();
            }

        } else if (type.equals("xlsx")) {
            try {
                FileInputStream file = new FileInputStream(getAbsolutePath());
                XSSFWorkbook workbook = new XSSFWorkbook(file);
                return workbook.getNumberOfSheets();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return -1;
    }

    public ArrayList<String> getColumn(int i) {
        try {
            ArrayList<String> output = new ArrayList<String>();
            Iterator<Row> rowIterator = null;
            if (type.equals("xls")) {
                FileInputStream file = new FileInputStream(getAbsolutePath());
                HSSFWorkbook workbook = new HSSFWorkbook(file);
                HSSFSheet sheet = workbook.getSheetAt(Input.worksheet);
                rowIterator = sheet.iterator();
            } else if (type.equals("xlsx")) {
                FileInputStream file = new FileInputStream(getAbsolutePath());
                XSSFWorkbook workbook = new XSSFWorkbook(file);
                XSSFSheet sheet = workbook.getSheetAt(Input.worksheet);
                rowIterator = sheet.iterator();
            }

            int previousCell;
            int currentCell;
            boolean skippedFirstRow = false;
            HashSet<Integer> rowLengths = new HashSet<Integer>();
            while (rowIterator.hasNext()) {
                previousCell = -1;
                //System.out.println("previous cell is -1");
                currentCell = 0;
                Row row = rowIterator.next();

                Iterator<Cell> cellIterator = row.cellIterator();
                int count = 0;
                for(int cn=0; cn<row.getLastCellNum(); cn++) {

                    
                    Cell cell = row.getCell(cn, Row.CREATE_NULL_AS_BLANK);
                    currentCell = cell.getColumnIndex();


                    if(Config.input.hasHeaders() && !skippedFirstRow) {
                        skippedFirstRow = true;
                        break;
                    } else if(count == i) {
                        //System.out.println("cell: " + cell);
                        //System.out.println(previousCell + " " + (currentCell-1));
                        /*if (previousCell != currentCell-1)  {
                            //then a blank was found
                            System.out.println("blank was found!");
                            break;
                        }
                        */
                        output.add(cell.toString());
                        //output.add(getAsString(cell));
                    }
                    count++;   
                    
                    previousCell = currentCell;
                }
                rowLengths.add(count);
                //System.out.println(output);
                if((Config.input.hasHeaders() && rowLengths.size() > 2) || (!Config.input.hasHeaders() && rowLengths.size() > 1)) {
                    //this means there are rows with different lengths
                    return null;
                }
                
            }
            
            return output;
            
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        System.out.println("r1");
        return null;
    }
    
    public int posOfHeader(String h) {
        getHeaders();
        for (int i = 0; i < headers.length; i++) {
            if (headers[i].equals(h)) {
                return i;
            }
        }
        return -1;
    }

    public ArrayList<String> getColumnByHeader(String header) {
        return getColumn(posOfHeader(header));
    }

    public ArrayList<String> getColumnByColumnAlphabet(String alpha) {
        return getColumn(posOfColumnInColumnAlphabet(alpha));
    }

    public int posOfColumnInColumnAlphabet(String alpha) {
        String alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

        ArrayList<String> all = new ArrayList<String>();
        for (int i = 0; i < alphabet.length(); i++) {
            all.add(alphabet.charAt(i) + "");
        }

        for (int i = 0; i < alphabet.length(); i++) {
            for (int j = 0; j < alphabet.length(); j++) {
                all.add(alphabet.charAt(i) + "" + alphabet.charAt(j) + "");
            }
        }

        return all.indexOf(alpha);
    }

    public String validateExpectedObserved() {
        if (Config.input.hasHeaders()) {
            exp = getColumnByHeader(Input.expectedColumn);
            obs = getColumnByHeader(Input.observedColumn);
                    
        } else {
            exp = getColumnByColumnAlphabet(Input.expectedColumn);
            obs = getColumnByColumnAlphabet(Input.observedColumn);
        }
        return Input.validateExpectedObserved(exp,obs);
    }
    
    public ArrayList<String> getExp() {
        return exp;
    }
    
    public ArrayList<String> getObs() {
        return obs;
    }
}
